/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Clippings extension for Mozilla Firefox.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005-2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// This source file is shared by both new.xul and newFolder.xul
//

var dialogArgs = window.arguments[0];
var gFolderMenu, gStrBundle;
var gClippingsSvc, gAppUtils;
var gSelectedFolderURI;
var gIsFolderMenuSeparatorInitialized = false;

// Used in new.xul only
var gClippingName, gClippingText;
var gIsFolderCreated;

// Used in newFolder.xul only
var gFolderName;


    
function initDlg() 
{
  try {
    gClippingsSvc = Components.classes["clippings@mozdev.org/clippings;1"].getService(Components.interfaces.nsIClippingsService);
  }
  catch (e) {
    alert(e);
    window.close();
  }

  gAppUtils = new aeClippingsUtils();
  gStrBundle = document.getElementById("ae-clippings-strings");

  var dsURL = gAppUtils.getCurrentProfileURL() + gAppUtils.CLIPDAT_FILE_NAME;
  try {
    var ds = gClippingsSvc.getDataSource(dsURL);
  }
  catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
    alert(gStrBundle.getString("errorOutOfMemory"));
    return;
  }
  catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
    alert(String.ae_fmt("%s: %s", gStrBundle.getString("errorAccessDenied"), gAppUtils.CLIPDAT_FILE_NAME));
    return;
  }
  catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
    alert(String.ae_fmt("%s: %s", gStrBundle.getString("errorFileLocked"),
			gAppUtils.CLIPDAT_FILE_NAME));
    return;
  }
  catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
    alert(String.ae_fmt("%s: %s", gStrBundle.getString("errorFileTooBig"),
			gAppUtils.CLIPDAT_FILE_NAME));
    return;
  }
  catch (e) {
    window.setTimeout("gAppUtils.alertEx(gStrBundle.getString('appName'),gStrBundle.getString('errorRecoveryIncomplete'));window.close()", 1);
  }

  gFolderMenu = document.getElementById("folder-menu-button");
  gFolderMenu.database.AddDataSource(ds);
  gFolderMenu.builder.rebuild();

  // newFolder.xul
  if (window.location.href == "chrome://clippings/content/newFolder.xul") {
    gFolderName = document.getElementById("folder-name");
    gFolderName.value = gStrBundle.getString('newFolderName');
    selectFolder(dialogArgs.parentFolderURI);
  }
  // new.xul
  else {
    gClippingName = document.getElementById("clipping-name");
    gClippingText = document.getElementById("clipping-text");
    gClippingName.value = dialogArgs.name;
    gClippingText.value = dialogArgs.text;
    gIsFolderCreated = false;
    gSelectedFolderURI = gClippingsSvc.kRootFolderURI;
  }
}


function checkForChangedFolders()
{
  if (isFolderMissing(gSelectedFolderURI)) {
    gAppUtils.log("Folder does not exist.  Defaulting to root folder.");
    gSelectedFolderURI = gClippingsSvc.kRootFolderURI;
    gFolderMenu.label = gStrBundle.getString("clippingsRoot");
    gFolderMenu.style.listStyleImage  = "url('chrome://clippings/skin/images/clippings-root.png')";
  }  
}


function isFolderMissing(aFolderURI)
{
  var rv = false;
  var exists;

  try {
    exists =  gClippingsSvc.exists(aFolderURI);
  }
  catch (e) {}

  if (! exists) {
    rv = true;
  }
  else {
    // Folders that exist, but have a detached parent folder, also qualify
    // as "missing."
    var parentURI;
    try {
      parentURI = gClippingsSvc.getParent(aFolderURI);
    }
    catch (e) {
      rv = true;
    }

    while (!rv && parentURI && parentURI != gClippingsSvc.kRootFolderURI) {
      if (gClippingsSvc.isDetached(parentURI)) {
	rv = true;
      }

      try {
	parentURI = gClippingsSvc.getParent(parentURI);
      }
      catch (e) {
	rv = true;
      }
    }
  }

  return rv;
}


function selectFolder(aFolderURI)
{
  gSelectedFolderURI = aFolderURI;
  if (aFolderURI == gClippingsSvc.kRootFolderURI) {
    gFolderMenu.label = gStrBundle.getString("clippingsRoot");
    gFolderMenu.style.listStyleImage = "url('chrome://clippings/skin/images/clippings-root.png')";
  }
  else {
    gFolderMenu.label = gClippingsSvc.getName(aFolderURI);
    gFolderMenu.style.listStyleImage = "url('chrome://clippings/skin/images/folder.png')";
  }
}


function initMenuSeparator(aMenuPopup)
{
  // Always rebuild folder menu separator
  var popup = document.getElementById("folder-menu-popup");
  var oldSep = document.getElementById("clippings-root-separator");
  if (oldSep) {
    popup.removeChild(oldSep);
  }

  if (gClippingsSvc.getCountSubfolders(gClippingsSvc.kRootFolderURI) > 0) {
    var clippingsRootMnuItem = document.getElementById("clippings-root");
    var newSep = document.createElement("menuseparator");
    newSep.id = "clippings-root-separator";
    aMenuPopup.insertBefore(newSep, clippingsRootMnuItem.nextSibling);

    gIsFolderMenuSeparatorInitialized = true;
  }
}


// new.xul only
function createFolder()
{
  var dlgArgs = { 
    parentFolderURI: gSelectedFolderURI || gClippingsSvc.kRootFolderURI
  };

  // Temporarily disable widgets in New Clipping dialog while New Folder dialog
  // is open.
  var okBtn = document.documentElement.getButton("accept");
  var cancelBtn = document.documentElement.getButton("cancel");
  var dlgElts = document.getElementsByTagName("*");

  for (var i = 0; i < dlgElts.length; i++) {
    dlgElts[i].disabled = true;
  }
  okBtn.disabled = true;
  cancelBtn.disabled = true;

  window.openDialog("chrome://clippings/content/newFolder.xul", "newfldr_dlg", "dialog,modal,centerscreen", dlgArgs);

  // After New Folder dialog is dismissed, re-enable New Clipping dlg widgets.
  for (var j = 0; j < dlgElts.length; j++) {
    dlgElts[j].disabled = false;
  }
  okBtn.disabled = false;
  cancelBtn.disabled = false;

  if (dlgArgs.userCancel) {
    return;
  }

  // Remove the separator following the Clippings root folder item in
  // preparation for the folder menu rebuild.
  var popup = document.getElementById("folder-menu-popup");
  var sep = document.getElementById("clippings-root-separator");
  if (sep) {
    popup.removeChild(sep);
    gIsFolderMenuSeparatorInitialized = false;
  }

  gFolderMenu.builder.rebuild();
  selectFolder(dlgArgs.newFolderURI);
  gIsFolderCreated = true;
}


function doOK() 
{
  if (! gSelectedFolderURI) {
    gSelectedFolderURI = gClippingsSvc.kRootFolderURI;
  }
  else {
    checkForChangedFolders();
  }

  // newFolder.xul
  if (window.location.href == "chrome://clippings/content/newFolder.xul") {
    var name = gFolderName.value;
    var uri = gClippingsSvc.createNewFolderEx(gSelectedFolderURI, null, name, null, false, gClippingsSvc.ORIGIN_NEW_CLIPPING_DLG);
    dialogArgs.newFolderURI = uri;
  }
  // new.xul
  else {
    dialogArgs.name = gClippingName.value;
    dialogArgs.text = gClippingText.value;
    dialogArgs.destFolder = gSelectedFolderURI;
  }

  dialogArgs.userCancel = false;
  return true;
}


function doCancel() 
{
  if (window.location.href == "chrome://clippings/content/new.xul"
      && gIsFolderCreated) {
    dialogArgs.destFolder = gSelectedFolderURI;
  }

  dialogArgs.userCancel = true;
  return true;
}
