/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Clippings extension for Mozilla Firefox.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005-2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Integration with host application
//


var gClippingsExec = {
  dataSrcInitialized: false,
  showDialog:    true,
  appUtils:      null,
  clippingsSvc:  null,
  strBundle:     null,


  newFromClipboard: function () 
  {
    if (! this.dataSrcInitialized) {
      alert("ERROR: Clippings hasn't been initialized yet!");
      return;
    }

    var txt = this.appUtils.getTextFromClipboard();
    if (! txt) {
      alert(this.strBundle.getString('errorNoClipboardContent'));
      return;
    }

    var result = this.clippingsSvc.newClippingFromText(txt, this.showDialog, window, null, false);
    if (result) {
      window.setTimeout("gClippingsExec.saveClippings();", 1);
    }
  },


  getSelectedText: function ()
  {
    var rv;
    var focusedWnd = document.commandDispatcher.focusedWindow;
    rv = focusedWnd.getSelection().toString();
    return rv;
  },


  openClippingsManager: function () 
  {
    var wnd = window.open("chrome://clippings/content/clippings.xul",
			  "clippings_wndobj", "chrome,resizable");
    wnd.focus();
  },


  insertClippingText: function (aText) 
  {
    this.appUtils.copyTextToClipboard(aText);

    // Paste clipping.  The following function is defined in
    // "chrome://global/content/globalOverlay.js"
    goDoCommand("cmd_paste");
    // SIDE EFFECT: The clipping text will remain on the system clipboard.
  },


  saveClippings: function () 
  {
    var title = this.strBundle.getString('appName');
    try {
      this.clippingsSvc.flushDataSrc();
    }
    catch (e if e.result == Components.results.NS_ERROR_NOT_INITIALIZED) {
      this.appUtils.alertEx(title, this.strBundle.getString("errorSaveFailedDSNotInitialized"));
      return;
	   }
    catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
      this.appUtils.alertEx(title, this.strBundle.getString("errorOutOfMemory"));
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorAccessDenied"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorFileLocked"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorFileTooBig"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_READ_ONLY) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString('errorFileReadOnly'),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_DISK_FULL) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString('errorDiskFull'),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e) {
      this.appUtils.alertEx(title, this.strBundle.getString("alertSaveFailed"));
      return;
    }
  },


  //
  // Browser window and Clippings menu initialization
  //

  isClippingsInitialized: false,

  initClippings: function ()
  {  
    // Workaround to this init function being called multiple times.
    if (this.isClippingsInitialized) {
      return;
    }

    this.appUtils = new aeClippingsUtils();
    this.strBundle = document.getElementById("ae-clippings-strings");

    try {
      this.clippingsSvc = Components.classes["clippings@mozdev.org/clippings;1"].getService(Components.interfaces.nsIClippingsService);
    }
    catch (e) {
      alert(e);
    }

    if (this.appUtils.getPref("clippings.first_run") == true) {
      this._firstRunInit();
      this.appUtils.setPref("clippings.first_run", false);
    }

    var err = false;
    var dsURL = this.appUtils.getCurrentProfileURL() + this.appUtils.CLIPDAT_FILE_NAME;
    try {
      var ds = this.clippingsSvc.getDataSource(dsURL);
    }
    catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
      err = this.strBundle.getString("errorOutOfMemory");
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorAccessDenied"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorFileLocked"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorFileTooBig"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e) {
      // File is corrupt - open Clippings Manager and perform recovery.
      err = true;
    }

    if (err) {
      this.openClippingsManager();
      return;
    }

    // Clippings backup
    var backupDirURL = this.appUtils.getCurrentProfileURL() + this.appUtils.BACKUP_DIR_NAME;
    this.clippingsSvc.setBackupDir(backupDirURL);
    this.clippingsSvc.setMaxBackupFiles(this.appUtils.getPref('clippings.backup.maxfiles'));

    gClippingsExec.dataSrcInitialized = true;

    this.appUtils.log("gClippingsExec.initClippings(): Clippings data source successfully loaded." + "\nHost app: " + this.appUtils.getHostAppName() + " (version " + this.appUtils.getHostAppVersionStr() + ")\nInitializing Clippings integration with host app window:\n" + window.location.href);

    this.showDialog = !(this.appUtils.getPref("clippings.entries.add_silently"));
    this.isClippingsInitialized = true;

    // **NOTE: Dynamically create status bar icon popup menu.  Defining it in
    // tbMessengerOverlay.xul won't work - it seems to disappear during loading
    // of the Messenger window.
    var popup = document.createElement("popup");
    popup.id = "ae-clippings-popup";

    var openClippingsMgr = document.createElement("menuitem");
    openClippingsMgr.setAttribute("label", this.strBundle.getString("openClippingsMgr"));
    openClippingsMgr.setAttribute("command", "ae_clippings_manager");
    openClippingsMgr.setAttribute("default", "true");

    var newFromClpbd = document.createElement("menuitem");
    newFromClpbd.setAttribute("command", "ae_new_clipping_from_clpbd");
    
    popup.appendChild(openClippingsMgr);
    popup.appendChild(newFromClpbd);
    document.getElementById("messengerWindow").appendChild(popup);

    // Initialize "New From Clipboard" command on status bar icon menu.
    var ellipsis = this.showDialog ? this.strBundle.getString("ellipsis") : "";
    var newFromClpbdCmd = document.getElementById("ae_new_clipping_from_clpbd");
    newFromClpbdCmd.setAttribute("label",
				 this.strBundle.getString("newFromClipbd")
				 + ellipsis);
    // Preloading icon images
    this._image1 = document.createElement("image");
    this._image2 = document.createElement("image");
    this._image1.src = "chrome://clippings/skin/images/folder.png";
    this._image2.src = "chrome://clippings/skin/images/clippings-icon.png";
  },

  _image1: null,
  _image2: null,

  _firstRunInit: function () {},

  unload: function () {}
};

// Keep in mind Gecko bug 241518 - calling addEventListener with a closure 
// holding a content node leaks the document (fixed in Firefox 1.6a1)
window.addEventListener("load", function () {gClippingsExec.initClippings()}, false);



// Drag 'n drop support
var ae_clippingsStatusBarObserver = {
  onDragOver: function (event, flavour, session)
  {
    event.target.setAttribute("ae-dnd", "true");
  },

  onDragExit: function (event, session)
  {
    event.target.removeAttribute("ae-dnd");
  },

  onDrop: function (event, transferData, session)
  {
    if (! gClippingsExec.dataSrcInitialized) {
      alert("ERROR: Clippings hasn't been initialized yet!");
      return;
    }

    var text = transferData.data;
    var result = gClippingsExec.clippingsSvc.newClippingFromText(text, gClippingsExec.showDialog, window, null, false);
    if (result) {
      window.setTimeout("gClippingsExec.saveClippings();", 100);
    }
  },

  getSupportedFlavours: function ()
  {
    var flavours = new FlavourSet();
    flavours.appendFlavour("text/unicode");
    return flavours;
  }
};


