/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Clippings extension for Mozilla Firefox.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005-2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Integration with host application
//


var gClippingsExec = {
  dataSrcInitialized: false,
  showDialog:    true,
  appUtils:      null,
  clippingsSvc:  null,
  strBundle:     null,
  listener:      null,

  newFromClipboard: function () 
  {
    if (! this.dataSrcInitialized) {
      // The RDF data source has to be initialized if it has not already been
      // done so, otherwise RDF node creation will fail, and the new Clippings
      // entry will never be created.
      // This initialization is done in the code for the Clippings popup menu's
      // `onpopupshowing' event handler.
      this.initClippingsPopup(document.getElementById("ae-clippings-popup-1"),
			      document.getElementById("ae-clippings-menu-1"));
    }

    var txt = this.appUtils.getTextFromClipboard();
    if (! txt) {
      alert(this.strBundle.getString('errorNoClipboardContent'));
      return;
    }

    var result = this.clippingsSvc.newClippingFromText(txt, this.showDialog, window, null, false);
    if (result) {
      window.setTimeout("gClippingsExec.saveClippings();", 1);
    }
  },


  getSelectedText: function ()
  {
    var rv;
    var focusedWnd = document.commandDispatcher.focusedWindow;
    rv = focusedWnd.getSelection().toString();
    return rv;
  },

  newFromSelection: function ()
  {
    if (! this.dataSrcInitialized) {
      this.initClippingsPopup(document.getElementById("ae-clippings-popup-1"),
			      document.getElementById("ae-clippings-menu-1"));
    }

    // Must explicitly close the message compose context menu - otherwise it
    // will reappear while the New Clipping dialog is open if the Clippings 
    // submenu needs to be rebuilt.
    var cxtMenu = document.getElementById("msgComposeContext");
    cxtMenu.hidePopup();

    var selection = this.getSelectedText();
    if (selection) {
      var result = this.clippingsSvc.newClippingFromText(selection, this.showDialog, window, null, false);
      if (result) {
	window.setTimeout("gClippingsExec.saveClippings();", 1);
      }
    }
    else {
      var strBundle = document.getElementById("ae-clippings-strings");
      alert(this.strBundle.getString('errorNoSelection'));
    }
  },

  openClippingsManager: function () 
  {
    var wnd = window.open("chrome://clippings/content/clippings.xul",
			  "clippings_wndobj", "chrome,resizable");
    wnd.focus();
  },


  initClippingsPopup: function (aPopup, aMenu) 
  {
    var err = false;
    var dsURL = this.appUtils.getCurrentProfileURL() + this.appUtils.CLIPDAT_FILE_NAME;
    try {
      var ds = this.clippingsSvc.getDataSource(dsURL);
    }
    catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
      err = this.strBundle.getString("errorOutOfMemory");
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorAccessDenied"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorFileLocked"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorFileTooBig"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e) {
      // File is corrupt - open Clippings Manager and perform recovery.
      err = 888;
    }

    var errorMenuItem = document.getElementById("ae-clippings-error-menuitem");

    if (err) {
      // Append a new menu item that lets user see more details of error.
      if (errorMenuItem) {
	return;
      }

      var errorCmd = document.createElement("menuitem");
      errorCmd.id = "ae-clippings-error-menuitem";
      errorCmd.setAttribute("label", this.strBundle.getString("errorMenu"));
      errorCmd.style.fontWeight = "bold";

      var func;
      if (err == 888) {
	func = function () { gClippingsExec.openClippingsManager() };
      }
      else {
	func = function () {
	  gClippingsExec.appUtils.alertEx(gClippingsExec.strBundle.getString('appName'), err);
	};
      }
      errorCmd.addEventListener("command", func, false);
      aPopup.appendChild(errorCmd);
      return;
    }
    else {
      // Remove error menu item if error condition no longer exists
      if (errorMenuItem) {
	aPopup.removeChild(errorMenuItem);
      }
    }

    aMenu.database.AddDataSource(ds);
    aMenu.builder.rebuild();

    this.dataSrcInitialized = true;
    this.appUtils.log("gClippingsExec.initClippingsPopup(): Data source initialization completed.");
  },

  insertClippingText: function (aText) 
  {
    this.appUtils.copyTextToClipboard(aText);

    // Paste clipping.  The following function is defined in
    // "chrome://global/content/globalOverlay.js"
    goDoCommand("cmd_paste");
    // SIDE EFFECT: The clipping text will remain on the system clipboard.
  },


  saveClippings: function () 
  {
    var title = this.strBundle.getString('appName');
    try {
      this.clippingsSvc.flushDataSrc();
    }
    catch (e if e.result == Components.results.NS_ERROR_NOT_INITIALIZED) {
      this.appUtils.alertEx(title, this.strBundle.getString("errorSaveFailedDSNotInitialized"));
      return;
	   }
    catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
      this.appUtils.alertEx(title, this.strBundle.getString("errorOutOfMemory"));
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorAccessDenied"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorFileLocked"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorFileTooBig"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_READ_ONLY) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString('errorFileReadOnly'),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_DISK_FULL) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString('errorDiskFull'),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e) {
      this.appUtils.alertEx(title, this.strBundle.getString("alertSaveFailed"));
      return;
    }
  },


  //
  // Browser window and Clippings menu initialization
  //

  isClippingsInitialized: false,

  initClippings: function ()
  {   
    // Workaround to this init function being called multiple times
    if (this.isClippingsInitialized) {
      return;
    }

    this.appUtils = new aeClippingsUtils();
    this.strBundle = document.getElementById("ae-clippings-strings");

    try {
      this.clippingsSvc = Components.classes["clippings@mozdev.org/clippings;1"].getService(Components.interfaces.nsIClippingsService);
    }
    catch (e) {
      alert(e);
    }

    if (this.appUtils.getPref("clippings.first_run") == true) {
      this._firstRunInit();
      this.appUtils.setPref("clippings.first_run", false);
    }

    // Clippings backup
    var backupDirURL = this.appUtils.getCurrentProfileURL() + this.appUtils.BACKUP_DIR_NAME;
    this.clippingsSvc.setBackupDir(backupDirURL);
    this.clippingsSvc.setMaxBackupFiles(this.appUtils.getPref('clippings.backup.maxfiles'));

    this.appUtils.log("gClippingsExec.initClippings(): Initializing Clippings integration with host app window: " + window.location.href);

    // Attaching event handlers to context menu 
    var hostAppContextMenu = document.getElementById("msgComposeContext");
    hostAppContextMenu.addEventListener("popupshowing", function (e) {gClippingsExec.initContextMenuItem(e)}, false);
    hostAppContextMenu.addEventListener("popuphiding", function (e) {gClippingsExec.hideContextMenuItem(e)}, false);

    // Initializing data source on Clippings context menu
    var menu = document.getElementById("ae-clippings-menu-1");
    var popup = document.getElementById("ae-clippings-popup-1");
    this.initClippingsPopup(popup, menu);

    this.showDialog = !(this.appUtils.getPref("clippings.entries.add_silently"));
    this.isClippingsInitialized = true;

    this.listener = {
      origin:  this.clippingsSvc.ORIGIN_HOSTAPP,

      newFolderCreated:    function (aFolderURI) {},
      newClippingCreated:  function (aClippingURI) {},
      importDone:          function (aNumItems) {}
    };

    this.clippingsSvc.addListener(this.listener);

    // Initialize "New From Clipboard" command on status bar icon menu.
    var ellipsis = this.showDialog ? this.strBundle.getString("ellipsis") : "";
    var newFromClpbdCmd = document.getElementById("ae_new_clipping_from_clpbd");
    newFromClpbdCmd.setAttribute("label",
				 this.strBundle.getString("newFromClipbd")
				 + ellipsis);
    // Preloading icon images
    this._image1 = document.createElement("image");
    this._image2 = document.createElement("image");
    this._image1.src = "chrome://clippings/skin/images/folder.png";
    this._image2.src = "chrome://clippings/skin/images/clippings-icon.png";
  },

  _image1: null,
  _image2: null,

  _firstRunInit: function () {},


  initContextMenuItem: function (event)
  {
    if (event.target.id != "msgComposeContext") {
      return;
    }

    this.appUtils.log("gClippingsExec.initContextMenuItem(): Initializing Clippings submenu");

    var menu = document.getElementById("ae-clippings-menu-1");
    menu.builder.rebuild();

    var strBundle = document.getElementById("ae-clippings-strings");
    var ellipsis = this.showDialog ? this.strBundle.getString("ellipsis") : "";
    var clippingsMenu1 = document.getElementById("ae-clippings-menu-1");
    var addEntryCmd = document.getElementById("ae_new_clipping_from_selection");
    var selection = this.getSelectedText();
  
    addEntryCmd.setAttribute("disabled", selection == "");
    addEntryCmd.setAttribute("label", strBundle.getString("newFromSelect")
			     + ellipsis);
  },

  hideContextMenuItem: function (event) {},

  unload: function ()
  {
    this.clippingsSvc.removeListener(this.listener);
    this.listener = null;
  }
};

// Keep in mind Gecko bug 241518 - calling addEventListener with a closure 
// holding a content node leaks the document (fixed in Firefox 1.6a1)
window.addEventListener("load", function () {gClippingsExec.initClippings()}, false);
window.addEventListener("unload", function () {gClippingsExec.unload()},false);



// Drag 'n drop support
var ae_clippingsStatusBarObserver = {
  onDragOver: function (event, flavour, session)
  {
    event.target.setAttribute("ae-dnd", "true");
  },

  onDragExit: function (event, session)
  {
    event.target.removeAttribute("ae-dnd");
  },

  onDrop: function (event, transferData, session)
  {
    if (! gClippingsExec.dataSrcInitialized) {
      // The RDF data source has to be initialized if it has not already been
      // done so, otherwise RDF node creation will fail, and the new Clippings
      // entry will never be created.
      // This initialization is done in the code for the Clippings popup menu's
      // `onpopupshowing' event handler.
      gClippingsExec.initClippingsPopup(document.getElementById("ae-clippings-popup-1"), document.getElementById("ae-clippings-menu-1"));
    }

    var text = transferData.data;
    var result = gClippingsExec.clippingsSvc.newClippingFromText(text, gClippingsExec.showDialog, window, null, false);
    if (result) {
      window.setTimeout("gClippingsExec.saveClippings();", 100);
    }
  },

  getSupportedFlavours: function ()
  {
    var flavours = new FlavourSet();
    flavours.appendFlavour("text/unicode");
    return flavours;
  }
};


